/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.deploy;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfox.ioc.Component;
import org.jfox.ioc.ComponentContext;
import org.jfox.ioc.ComponentMeta;
import org.jfox.ioc.ComponentMetaFactory;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.Registry;
import org.jfox.ioc.classloader.LocalClassLoader;
import org.jfox.ioc.depend.ComponentRefDependency;
import org.jfox.ioc.depend.Dependency;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.depend.ObjectDependency;
import org.jfox.ioc.deployment.Deployable;
import org.jfox.ioc.deployment.Deployer;
import org.jfox.ioc.exception.ComponentException;
import org.jfox.ioc.util.Classes;

/**
 * deploy Component to Registry
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ComponentDeployer implements Deployer, Component {
    private Registry registry;

    // module => ComponentNames
    private Map modules = new HashMap();

    public Registry getRegistry() {
        return registry;
    }

    public void deploy(Deployable comp) throws DeployException {
        if(!(comp instanceof ComponentModule)) {
            throw new DeployException("can not deploy " + comp + ", it's not a instance of" + ComponentModule.class.getName());
        }
        ComponentModule componentModule = (ComponentModule) comp;
        ComponentModuleDescriptor md = componentModule.getModuleDescriptor();

        // load public classes
        String[] publicClasses = md.getPublicClasses();
        for(int i=0; i<publicClasses.length; i++ ){
            String pubClass = publicClasses[i];
            try {
                componentModule.getClassLoader().loadClass(pubClass);
            }
            catch(Exception e){
                throw new DeployException("failed to load module public class " + pubClass,e);
            }
        }

        ComponentDescriptor[] cds = md.getComponentDescriptors();
        List metas = new ArrayList(cds.length);
        for(int i = 0; i < cds.length; i++) {
            ComponentMeta meta = createComponentMeta(((LocalClassLoader) componentModule.getClassLoader()), cds[i], componentModule.getTempDeployDirectory());
            metas.add(meta);
        }

        ComponentMeta[] successMetas = registry.registerComponentBatch((ComponentMeta[])metas.toArray(new ComponentMeta[metas.size()]));

        List names = new ArrayList(cds.length);
        for(int i = 0; i < successMetas.length; i++) {
            ComponentName name = ((ComponentMeta)successMetas[i]).getComponentName();
            names.add(name);
        }
        modules.put(componentModule, names);
    }

    public void undeploy(Deployable comp) throws DeployException {
        if(!(comp instanceof ComponentModule)) {
            throw new DeployException("can not undeploy " + comp + ", it's not a instance of" + ComponentModule.class.getName());
        }
        ComponentModule componentModule = (ComponentModule) comp;
        if(!modules.containsKey(componentModule)) return;
        List names = (List) modules.get(componentModule);

        for(int i = names.size() - 1; i >= 0; i--) {
            ComponentName name = (ComponentName) names.get(i);
            try {
                registry.unregisterComponent(name);
            }
            catch(ComponentException e) {
                throw new DeployException("undeploy " + name + " faild.", e);
            }
        }
        modules.remove(componentModule);
    }

/*
    private ComponentName deploy(LocalClassLoader cl, ComponentDescriptor cd, File moduleDeployDir) throws DeployException {
        String description = cd.getDescription();
        String compImpl = cd.getImplementation();
        String compName = cd.getName();
        if(compName == null || compName.trim().equals("")){
            throw new DeployException("component name can not be null. it's implementation is " + compImpl);
        }
        boolean singleton = cd.isSingleton();
        boolean typeSingleton = cd.isTypeSingleton();
        int priority = cd.getPriority();
        ConstructorDescriptor constructorDesc = cd.getConstructorDescriptor();
        PropertyDescriptor[] propertyDesces = cd.getPropertyDescriptors();

        try {
            ComponentMeta meta = ComponentMetaFactory.getComponentMeta(ComponentName.parseString(compName, cl),
                    cl.loadClass(compImpl),
                    buildConstrParameter(cl, constructorDesc),
                    buildPropertyParameterPacks(cl,propertyDesces),
                    singleton,
                    typeSingleton
                    );
            meta.setPriority(priority);
            meta.setDescription(description);
            meta.setModuleDescription(cd.getModuleDescription());
            meta.setModule(moduleDeployDir.getPath());
            registry.registerComponent(meta);
            return meta.getComponentName();
        }
        catch(Exception e) {
            throw new DeployException(e);
        }

    }
*/

    private Dependency[] buildConstrParameter(ClassLoader cl, ConstructorDescriptor cd) throws ComponentException {
        if(cd == null) {
            // null will case the InstantiatingComponentFactory auto detect constructor
            return null;
        }
        ParameterDescriptor[] pds = cd.getParamDescriptors();
        Dependency[] params = new Dependency[pds.length];
        for(int i = 0; i < pds.length; i++) {
            params[i] = buildParameter(cl, pds[i]);
        }
        return params;
    }

    private DependencyPack[] buildPropertyParameterPacks(ClassLoader cl, PropertyDescriptor[] pds) throws ComponentException {
        DependencyPack[] ppps = new DependencyPack[pds.length];
        for(int i = 0; i < pds.length; i++) {
            ppps[i] = buildPropertyParameterPack(cl, pds[i]);
        }
        return ppps;
    }

    private DependencyPack buildPropertyParameterPack(ClassLoader cl, PropertyDescriptor pd) throws ComponentException {
        String name = pd.getName();
        ParameterDescriptor[] pds = pd.getParamDescriptors();
        Dependency param = buildParameter(cl, pds[0]);
        return new DependencyPack(name, param);
    }

    private Dependency buildParameter(ClassLoader cl, ParameterDescriptor pd) throws ComponentException {
        String ref = pd.getRef();
        if(ref != null && !ref.equals("")) {
            if(ref.indexOf("@") > 0) {
                return new ComponentRefDependency(ComponentName.parseString(ref, cl));
            }
            else {
                try {
                    return new ComponentRefDependency(ComponentName.newInstance(cl.loadClass(ref)));
                }
                catch(Exception e) {
                    throw new ComponentException("build parameter error.", e);
                }
            }
        }
        String type = pd.getType();
        String value = pd.getValue();
        try {
            Class typeClass = null;
            if(Classes.isPrimitiveClass(type)) {
                typeClass = Classes.loadPrimitiveClass(type);
            }
            else {
                typeClass = cl.loadClass(type);
            }
            Object param = Classes.newObject(typeClass, value);
            return new ObjectDependency(param, typeClass);
        }
        catch(Exception e) {
            throw new ComponentException("build parameter error.", e);
        }
    }

    public void setComponentContext(ComponentContext ctx) {
        this.registry = ctx.getRegistry();
    }


    private ComponentMeta createComponentMeta(LocalClassLoader cl, ComponentDescriptor cd, File moduleDeployDir) throws DeployException {
        String description = cd.getDescription();
        String compImpl = cd.getImplementation();
        String compName = cd.getName();
        if(compName == null || compName.trim().equals("")){
            throw new DeployException("component name can not be null. it's implementation is " + compImpl);
        }
        boolean singleton = cd.isSingleton();
        boolean typeSingleton = cd.isTypeSingleton();
        int priority = cd.getPriority();
        ConstructorDescriptor constructorDesc = cd.getConstructorDescriptor();
        PropertyDescriptor[] propertyDesces = cd.getPropertyDescriptors();

        try {
            ComponentMeta meta = ComponentMetaFactory.getComponentMeta(
                    ComponentName.parseString(compName, cl), cl
                            .loadClass(compImpl), buildConstrParameter(cl,
                            constructorDesc), buildPropertyParameterPacks(cl,
                            propertyDesces), singleton, typeSingleton);
            meta.setPriority(priority);
            meta.setDescription(description);
            meta.setModule(moduleDeployDir.getName());
            meta.setModuleDescription(cd.getModuleDescription());
            meta.setModuleDir(moduleDeployDir.getPath());
            return meta;
        }
        catch (Exception e) {
            throw new DeployException(e);
        }

    }

    public static void main(String[] args) {

    }
}

