/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */
package org.jfox.ioc.deploy;

import org.jfox.ioc.Constants;
import org.jfox.ioc.classloader.ClassLoaderRepository;
import org.jfox.ioc.classloader.LocalClassLoader;
import org.jfox.ioc.deployment.Deployable;
import org.jfox.ioc.deployment.Deployer;
import org.jfox.ioc.logger.Logger;
import org.jfox.ioc.util.FileFilters;
import org.jfox.ioc.util.Files;
import org.jfox.ioc.util.XMLUtils;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;

/**
 * a module directory
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ComponentModule implements Deployable, Comparable<ComponentModule> {
	private File moduleDir;

	private LocalClassLoader moduleClassLoader;

	private ClassLoaderRepository loaderRepo;

	private ComponentModuleDescriptor componentModuleDescriptor;

	private Logger logger;

	private static final String singleComponentDescriptorSuffix = ".component.xml";

	public ComponentModule(File moduleDir, ClassLoaderRepository loaderRespo)
			throws ComponentDescriptionException {
		this.moduleDir = moduleDir;
		this.loaderRepo = loaderRespo;
		logger = Logger.getLogger(this.getClass().getName() + "["
		                          + moduleDir.getName() + "]");
		init();
	}

	public File getPackage() {
		return moduleDir;
	}

	public void deploy(Deployer deployer) throws DeployException {
		deployer.deploy(this);
	}

	public File getTempDeployDirectory() {
		return moduleDir;
	}

	public void init() throws ComponentDescriptionException {
		try {
			componentModuleDescriptor = new ComponentModuleDescriptor(XMLUtils
					.getDocument(getDescriptorURL()));
		}
		catch(Exception e) {
			throw new ComponentDescriptionException("build module descriptor failed.", e);
		}

		// Ҷ component
		try {
			List components = Files.listFiles(new File(moduleDir, "META-INF"), FileFilters.suffixFileFilter(singleComponentDescriptorSuffix));
			for(int i = 0; i < components.size(); i++) {
				File componentXml = (File) components.get(i);
				ComponentDescriptor cd = new ComponentDescriptor(componentModuleDescriptor);
				cd.processXML(XMLUtils.getDocument(componentXml.toURL()).getDocumentElement());
				componentModuleDescriptor.addComponent(cd);
			}

		}
		catch(Exception e) {
			throw new ComponentDescriptionException("build module descriptor failed.", e);
		}

		// add component jars to Common loader
		File[] componentJars = getComponentJars();
		List urls = new ArrayList(componentJars.length + 1);
		try {
			// add the classes dir to pub class path
			File classesDir = new File(moduleDir, Constants.JFOXCO_MODULE_ClASSPATH);
			if(classesDir.exists() && classesDir.isDirectory()) {
				urls.add(classesDir.toURL());
				logger
						.debug("add component classes directory to module's public ClassLoader "
						       + classesDir.toString());
			}

			for(int i = 0; i < componentJars.length; i++) {
				urls.add(componentJars[i].toURL());
				logger
						.debug("add component jar to module's public ClassLoader "
						       + componentJars[i].toString());
			}

			// add resource dir
			File resourceDir = new File(moduleDir, Constants.JFOXCO_MODULE_CONFIG_PATH);
			if(resourceDir.exists() && resourceDir.isDirectory()) {
				urls.add(resourceDir.toURL());
				logger
						.debug("add module resource directory to module's public ClassLoader "
						       + resourceDir.toString());
			}

		}
		catch(Exception e) {
			e.printStackTrace();
		}
		// constructor's urls is the module's pub jars
		moduleClassLoader = new LocalClassLoader((URL[]) urls
				.toArray(new URL[urls.size()]), loaderRepo);

		// add component lib jars to module private loader
		File[] libJars = getComponentLibJars();
		try {
			for(int i = 0; i < libJars.length; i++) {
				moduleClassLoader.addURL(libJars[i].toURL());
				logger
						.debug("add component lib to module's private ClassLoader "
						       + libJars[i].toString());
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	public URL getDescriptorURL() {
		try {
			return new File(moduleDir, "META-INF/" + Constants.JFOXCO_MODULE_DEPLOY_FILE).toURL();
		}
		catch(Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public URLClassLoader getClassLoader() {
		return moduleClassLoader;
	}

	public ComponentModuleDescriptor getModuleDescriptor() {
		return componentModuleDescriptor;
	}

	public File[] getComponentJars() {
		return moduleDir.listFiles(FileFilters.suffixFileFilter(new String[]{
				"jar", "zip"}));
	}

	private File[] getComponentLibJars() {
		File lib = new File(moduleDir, Constants.JFOXCO_MODULE_PRIVATE_LIBS);
		if(!lib.exists())
			return new File[0];
		List files = Files.listFiles(lib, FileFilters
				.suffixFileFilter(new String[]{"jar", "zip"}));
		return (File[]) files.toArray(new File[files.size()]);
	}

	public int compareTo(ComponentModule o) {
		ComponentModule _module = (ComponentModule) o;
		return new Integer(componentModuleDescriptor.getPriority())
				.compareTo(new Integer(_module.getModuleDescriptor()
						.getPriority()));
	}

	public boolean equals(Object obj) {
		return moduleDir.getName().equals(((ComponentModule) obj).moduleDir.getName());
	}

	public int hashCode() {
		return moduleDir.getName().toString().hashCode();
	}

	public String toString() {
		return "Module{" + "moduleDir=" + moduleDir + ", description="
		       + componentModuleDescriptor.getDescription() + "}";
	}

	public static void main(String[] args) {

	}
}

