/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */
package org.jfox.deployment.ejb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.ejb.EJBHome;
import javax.ejb.EJBMetaData;
import javax.ejb.EJBObject;
import javax.ejb.EnterpriseBean;
import javax.jms.MessageListener;

import org.jfox.ejb.BucketMeta;
import org.jfox.ejb.BucketMetaImpl;
import org.jfox.ejb.EJBMetaDataImpl;
import org.jfox.ejb.connector.EJBContainerImpl;
import org.jfox.ejb.meta.EJBBundle;
import org.jfox.ejb.meta.EJBDescriptor;
import org.jfox.ejb.meta.MDBDescriptor;
import org.jfox.ejb.meta.SessionDescriptor;
import org.jfox.ioc.common.AbstractComponent;
import org.jfox.ioc.connector.Container;
import org.jfox.ioc.deploy.DeployException;
import org.jfox.ioc.deployment.Deployable;
import org.jfox.ioc.deployment.Deployer;
import org.jfox.ioc.logger.Logger;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class EJBDeployer extends AbstractComponent implements Deployer {
    private EJBContainerImpl container;
    private Logger logger = Logger.getLogger(getClass().getName());

    // ejbModule => ejbDescriptor List
    private Map deployedEjbs = new HashMap();

//  private static EJBDeployer instance = new EJBDeployer(ContainerImpl.getInstance());

    public EJBDeployer(Container ejbContainer) {
        this.container = (EJBContainerImpl)ejbContainer;
    }

    public void deploy(Deployable comp) throws DeployException {
        if(!(comp instanceof EJBModule)) {
            throw new DeployException("can not deploy non ejb module");
        }
        EJBModule ejbModule = (EJBModule) comp;
        // redeploy, do undeploy first
        if(deployedEjbs.containsKey(ejbModule)) {
            undeploy(ejbModule);
        }

        ClassLoader loader = ejbModule.getClassLoader();
        EJBBundle ejbBundle = ejbModule.getEJBBundle();
        EJBDescriptor[] ejbDescriptors = ejbBundle.getEjbDescriptors();
        for(int i = 0; i < ejbDescriptors.length; i++) {
            EJBDescriptor ejbDescriptor = ejbDescriptors[i];
            try {
                // load EJBMeta as addition description
//                loadEjbMeta(loader, ejbDescriptor);
                // try to deploy ejb
                deploy(loader, ejbDescriptor);
                addDeployedEjb(ejbModule, ejbDescriptor);
            }
            catch(Exception e) {
                logger.warn(e);
            }
        }
    }

    protected void doInit() throws Exception {
    }

    protected void doDestroy() throws Exception {
    }

    private void addDeployedEjb(EJBModule ejbModule, EJBDescriptor ejbDescriptor) {
        if(!deployedEjbs.containsKey(ejbModule)) {
            List ejbDesriptors = new ArrayList();
            ejbDesriptors.add(ejbDescriptor);
            deployedEjbs.put(ejbModule, ejbDesriptors);
        }
        else {
            List ejbDescriptors = (List) deployedEjbs.get(ejbModule);
            ejbDescriptors.add(ejbDescriptor);
        }
    }

    private void deploy(ClassLoader loader, EJBDescriptor ejbDescriptor) throws DeployException {
        if(ejbDescriptor instanceof SessionDescriptor){
            deploySession(loader,(SessionDescriptor)ejbDescriptor);
        }
        else if(ejbDescriptor instanceof MDBDescriptor){
            deployMDB(loader,(MDBDescriptor)ejbDescriptor);
        }
        else {
            throw new DeployException(" unkonw ejb desriptor type " + ejbDescriptor);
        }

    }

    private void deploySession(ClassLoader loader, SessionDescriptor sessionDescriptor) throws DeployException {

        try {
            Class homeClass = loader.loadClass(sessionDescriptor.getHomeClassName());
            loader.loadClass("javax.ejb.EJBHome");
            if(!testHomeClass(homeClass)) {
                logger.warn("Home Class: " + homeClass + " not extends " + EJBHome.class.getName());
            }

            Class remoteClass = loader.loadClass(sessionDescriptor.getRemoteClassName());
            if(!testRemoteClass(remoteClass)) {
                logger.warn("Bean Interface Class: " + remoteClass + " not extends " + EJBObject.class.getName());
            }

            Class beanClass = loader.loadClass(sessionDescriptor.getEjbClassName());
            if(!testBeanClass(beanClass)) {
                logger.warn("Bean Class: " + beanClass + " not extends " + EnterpriseBean.class.getName());
            }

            EJBMetaData ejbMetaData = null;
            if(sessionDescriptor.getType().equals(EJBDescriptor.SESSION_BEAN)) { // is SessionBean
                ejbMetaData = new EJBMetaDataImpl(homeClass, remoteClass, true, ((SessionDescriptor) sessionDescriptor).isStateless());
            }
            else { // Maybe EntityBean, but not support now
                ejbMetaData = new EJBMetaDataImpl(homeClass, remoteClass, true, false);
            }
            // will create a ejb bucket and bind it's home to jndi
            BucketMeta bucketMeta = new BucketMetaImpl(ejbMetaData, sessionDescriptor, beanClass);
            container.createBucket(bucketMeta);
            logger.info("deploy SessionBean " + beanClass.getName() + " successful");
        }
        catch(Exception e) {
            e.printStackTrace();
            throw new DeployException("deploy ejb " + sessionDescriptor.getEjbName() + " failed.", e);
        }
    }

    private void deployMDB(ClassLoader loader, MDBDescriptor mdbDescriptor) throws DeployException{
        try {
            Class beanClass = loader.loadClass(mdbDescriptor.getEjbClassName());
            if(!testMDBBeanClass(beanClass)){
                logger.warn("MDB Bean Class " + beanClass.getName() + " is invalid, maybe not implements " + EnterpriseBean.class.getName() + " and " + MessageListener.class.getName());
            }
            BucketMeta meta = new BucketMetaImpl(new EJBMetaDataImpl(null,null,false,false), mdbDescriptor,beanClass);
            container.createBucket(meta);
            logger.info("deploy MessageDriven Bean " + beanClass.getName() + " successful.");
        }
        catch(Exception e){
            throw new DeployException("deploy MessageDriven Bean " + mdbDescriptor.getEjbName() + " failed.", e);
        }
    }

    public void undeploy(Deployable comp) throws DeployException {
        EJBModule ejbModule = (EJBModule) comp;
        if(deployedEjbs.containsKey(ejbModule)) {
            List ejbDescriptors = (List) deployedEjbs.get(ejbModule);
            for(int i = 0; i < ejbDescriptors.size(); i++) {
                String ejbName = ((EJBDescriptor) ejbDescriptors.get(i)).getEjbName();
                try {
                    container.dropBucket(ejbName);
                    logger.debug("undeploy " + ejbName + " successful.");
                }
                catch(Exception e) {
                    logger.warn("undeploy " + ejbName + " failed.", e);
                }
            }
        }
        deployedEjbs.remove(ejbModule);
    }

    private boolean testHomeClass(Class homeClass) {
        return EJBHome.class.isAssignableFrom(homeClass);
    }

    private boolean testRemoteClass(Class remoteClass) {
        return EJBObject.class.isAssignableFrom(remoteClass);
    }

    private boolean testBeanClass(Class beanClass) {
        return EnterpriseBean.class.isAssignableFrom(beanClass);
    }

    private boolean testMDBBeanClass(Class beanClass){
        return (EnterpriseBean.class.isAssignableFrom(beanClass) && MessageListener.class.isAssignableFrom(beanClass));
    }
    public static void main(String[] args) {

    }
}

