/*
 * Copyright (c) 2004 Your Corporation. All Rights Reserved.
 */

package org.jfox.ejb.connector;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.naming.Context;
import javax.transaction.TransactionManager;

import org.jfox.ejb.Bucket;
import org.jfox.ejb.BucketMeta;
import org.jfox.ejb.BucketSupport;
import org.jfox.ejb.EJBObjectId;
import org.jfox.ejb.MDBBucket;
import org.jfox.ejb.StatefulBucket;
import org.jfox.ejb.StatelessBucket;
import org.jfox.ejb.handler.Interceptor;
import org.jfox.ejb.meta.MDBDescriptor;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.connector.AbstractContainer;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.depend.ObjectDependency;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.jndi.InitialContextHelper;
import org.jfox.tm.TxManager;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class EJBContainerImpl extends AbstractContainer implements EJBContainer, ActiveComponent {

    // jndi instance
    static Context INITIAL_CONTEXT = null;

    // loaded bean buckets
    private Map buckets = new HashMap();

//    private BucketFactory bucketFactory = BucketFactory.getInstance();

    private List interceptors = new ArrayList();

    public EJBContainerImpl() {

    }

    public Class getHandlerClass() {
        return EJBHandler.class;
    }

    public TransactionManager getTransactionManager() {
        return TxManager.getInstance();
    }

    public Context getInitialContext() {
        return INITIAL_CONTEXT;
    }

    protected void doInit() throws Exception {
        INITIAL_CONTEXT = InitialContextHelper.getInitialContext();
    }

    protected synchronized void doDestroy() throws Exception {
        for(Iterator it = buckets.values().iterator(); it.hasNext();) {
            ((BucketSupport) it.next()).destroy();
        }
        INITIAL_CONTEXT = null;
        buckets = new HashMap();
    }

    public Iterator listBuckets() {
        return buckets.keySet().iterator();
    }

    /**
     * create a bean bucket and bind it's home to jndi
     *
     * @param meta
     */
    public synchronized void createBucket(BucketMeta meta) throws Exception {
        Bucket bucket = null;
        if(meta.getEJBMetaData().isSession()) {
            if(meta.getEJBMetaData().isStatelessSession()) {
                bucket = (StatelessBucket)context.getRegistry().getComponentInstance(ComponentName.newInstance(Bucket.class,"Stateless"),
                                                                                     new DependencyPack[]{
                                                                                         new DependencyPack("bucketMeta",new ObjectDependency(meta))
                                                                                     });
            }
            else {
                bucket = (StatefulBucket) context.getRegistry().getComponentInstance(ComponentName.newInstance(Bucket.class, "Stateful"),
                                                                                     new DependencyPack[]{
                                                                                        new DependencyPack("bucketMeta", new ObjectDependency(meta))
                                                                                     });
            }
        }
        else if(meta.getEJBDescriptor() instanceof MDBDescriptor){
            bucket = (MDBBucket) context.getRegistry().getComponentInstance(ComponentName.newInstance(Bucket.class,"MDB"),
                                                                            new DependencyPack[]{
                                                                                new DependencyPack("bucketMeta",new ObjectDependency(meta))
                                                                            });
        }
        else {
            throw new RuntimeException("not support EntityBean now!");
        }
//        Bucket bucket = bucketFactory.createBucket(meta);
//        bucket.init();
        buckets.put(meta.getEJBDescriptor().getEjbName(), bucket);
    }

    /**
     * remove the bucket from container ,  undeploy һ bean ʱ
     */
    public synchronized void dropBucket(String ejbName) throws Exception {
        buckets.remove(ejbName);
        ((Bucket) buckets.get(ejbName)).destroy();
    }

    /**
     * container ܶԸ÷ýһЩͳһĵأ磺ȫ֤ȣȻ forward  bucket ִ
     *
     * @return
     * @throws Exception
     */
    public Object invoke(EJBInvocation invocation) throws Exception {
        logger.debug("invoke " + invocation.toString());
        EJBObjectId ejbObjectId = (EJBObjectId)invocation.getObjectId();
        String ejbName = ejbObjectId.getEjbName();
        Bucket bucket = getBucket(ejbName);

        if(((EJBObjectId)invocation.getObjectId()).isHome()) { // EJBHome 
            invocation.setMethod(bucket.getHomeMethod(invocation));
            preInvokeHome(invocation);
            Object result = bucket.invokeHome(invocation);
            postInvokeHome(invocation);
            return result;
        }
        else { // Bean 
            invocation.setMethod(bucket.getBeanMethod(invocation));
            preInvokeBean(invocation);
            Object result = bucket.invokeBean(invocation);
            postInvokeBean(invocation);
            return result;
        }

    }

    public void addInterceptor(Interceptor interceptor) throws Exception {
        interceptor.setContainer(this);
        synchronized(interceptors) {
            interceptors.add(interceptor);
        }
    }

    public synchronized Iterator interceptors() {
        return interceptors.iterator();
    }

    public Bucket getBucket(String ejbName) {
        return (Bucket) buckets.get(ejbName);
    }

    private void preInvokeHome(EJBInvocation invocation) throws Exception {
        for(Iterator it = this.interceptors(); it.hasNext();) {
            Interceptor interceptor = (Interceptor) it.next();
            interceptor.preInvokeHome(invocation);
        }
    }

    private void preInvokeBean(EJBInvocation invocation) throws Exception {
        for(Iterator it = this.interceptors(); it.hasNext();) {
            Interceptor interceptor = (Interceptor) it.next();
            interceptor.preInvokeBean(invocation);
        }
    }

    private void postInvokeHome(EJBInvocation invocation) throws Exception {
        for(Iterator it = this.interceptors(); it.hasNext();) {
            Interceptor interceptor = (Interceptor) it.next();
            interceptor.preInvokeHome(invocation);
        }
    }

    private void postInvokeBean(EJBInvocation invocation) throws Exception {
        for(Iterator it = this.interceptors(); it.hasNext();) {
            Interceptor interceptor = (Interceptor) it.next();
            interceptor.preInvokeBean(invocation);
        }
    }

    public static void main(String[] args) {

    }
}
